import { Component, h, Prop, Watch, State, Listen } from '@stencil/core';
import '@pie-framework/pie-player-components';
import ResizeObserver from 'resize-observer-polyfill';
import jsonBeautify from 'json-beautify';
import { getPackageWithoutVersion, packageToElementName } from '../../util/utils';
import classnames from 'classnames';
import docson from 'docson';
import range from 'lodash/range';
import isEqual from 'lodash/isEqual';
import cloneDeep from 'lodash/cloneDeep';
import merge from 'lodash/merge';
import throttle from 'lodash/throttle';
import debug from 'debug';
const log = debug('pie-framework:pie-demo');
var ViewState;
(function (ViewState) {
    ViewState[ViewState["LOADING"] = 0] = "LOADING";
    ViewState[ViewState["READY"] = 1] = "READY";
    ViewState[ViewState["ERROR"] = 2] = "ERROR";
})(ViewState || (ViewState = {}));
const defaultController = {
    model: (model, session, env) => {
        log(model, session, env);
        return Object.assign(Object.assign({}, model), { mode: env.mode });
    },
    outcome: () => ({
        score: 0,
        details: {}
    })
};
export class PieDemoContent {
    constructor() {
        /**
         * Removes control bar if true
         * @type {boolean}
         */
        this.justElement = false;
        /**
         * Makes the element have 50% height
         * @type {boolean}
         */
        this.multiplePies = false;
        /**
         * Link to the pie-schema markdown file
         * @type {boolean}
         */
        this.modelSchemaJSONURI = null;
        /**
         * Link to the pie-schema markdown file
         * @type {boolean}
         */
        this.configureSchemaJSONURI = null;
        /**
         * Tells the component if it needs to load the elements or not
         */
        this.load = true;
        /**
         * Include an editor in the view
         */
        this.editor = true;
        /**
         * Include an item preview in the view
         */
        this.preview = true;
        /**
         * Include control panel for adjusting player settings.
         */
        this.playerControls = true;
        this.state = ViewState.LOADING;
        // each key will be a package name, without version
        this.pieControllers = {};
        this.minHeightAuthoring = 'initial';
        this.minHeightStudent = 'initial';
        this.rootElWidth = 1250;
        this.bottomContElWidth = 550;
        this.tabIndex = 0;
        this.mobileTabIndex = 0;
        this.currentOption = 'student';
        this.studSettVisible = false;
        this.authSettVisible = false;
        this.docHolderVisible = false;
        this.env = { mode: 'gather' };
        this.sessions = {};
        this.scores = {};
        this._renderStudent = false;
        this.imageHandler = null;
        this.configureSchemaJSON = null;
        this.modelSchemaJSON = null;
        // todo replace cached Json config
        this.cachedJsonConfig1 = null;
        this.rootEl = null;
        this.fileInput = null;
        this._renderingAuthor = false;
        this._renderingStudent = false;
        this.getProperController = (model) => {
            const divA = document.createElement('div');
            if ('markup' in this.config) {
                divA.innerHTML = this.config.markup;
                if (model.id) {
                    const tag = Object.keys(this.config.elements).find(key => key === divA.querySelector(`[id="${model.id}"]`).tagName.toLowerCase());
                    const packageWithoutVersion = getPackageWithoutVersion(this.config.elements[tag]);
                    const controller = this.pieControllers[packageWithoutVersion];
                    return {
                        controller,
                        tag
                    };
                }
                return {};
            }
            return {};
        };
        this.viewDocumentation = tabIndex => {
            log('Tab Index', tabIndex);
            const currentContent = tabIndex === 1 ? this.modelSchemaJSON : this.configureSchemaJSON;
            this.docHolderVisible = true;
            docson.doc('schema-holder', currentContent);
        };
        this.renderJsonConfigPanel = (jsonData, index) => {
            log(jsonData);
            return (h("div", { class: "json-config" },
                h("div", { class: "json-config-header" },
                    h("div", { class: "view-container", onClick: () => this.viewDocumentation(index) },
                        h("i", { class: "fa fa-desktop" }),
                        h("span", null, "View Documentation")),
                    h("div", { class: "right-content" },
                        h("div", { class: "copy-container", onClick: () => {
                                this[`textAreaContentRef${index}`].select();
                                document.execCommand('copy');
                            } },
                            h("i", { class: "fa fa-clone" }),
                            h("span", null, "Copy JSON")),
                        h("div", { class: "reset-container", onClick: () => {
                                if (this[`textAreaContentRef${index}`]) {
                                    if (index === 1) {
                                        this[`textAreaContentRef${index}`].value = jsonBeautify(this[`cachedJsonConfig${index}`], null, 2, 98);
                                        this.config = this[`cachedJsonConfig${index}`];
                                    }
                                    else {
                                        this[`textAreaContentRef${index}`].value = jsonBeautify(this.configSettings, null, 2, 98);
                                        this.configureObject = this.configSettings;
                                    }
                                }
                            } },
                            h("i", { class: "fa fa-undo" }),
                            h("span", null, "Reset")))),
                h("textarea", { ref: el => (this[`textAreaContentRef${index}`] = el), class: "json-content", value: jsonBeautify(jsonData, null, 2, 98), onChange: e => {
                        const target = e.target;
                        if (index === 1) {
                            this.config = JSON.parse(target.value);
                        }
                        else {
                            this.configureObject = JSON.parse(target.value);
                        }
                    } })));
        };
        this.renderTabs = (tabs, tabWidth = 100) => {
            const content = tabs[this.tabIndex] ? tabs[this.tabIndex].content : null;
            const contentValue = typeof content === 'function' ? content() : content;
            return (h("div", { class: "tabs-container" },
                h("div", { class: "tabs" }, tabs.map((tab, index) => (h("div", { style: {
                        width: `${tabWidth}px`
                    }, class: classnames('tab', {
                        selected: this.tabIndex === index
                    }), onClick: () => (this.tabIndex = index) }, tab.name)))),
                h("span", { class: "selected-line", style: {
                        left: `${this.tabIndex * tabWidth}px`,
                        width: `${tabWidth}px`
                    } }),
                h("div", { class: "tab-content" }, contentValue)));
        };
        this.renderAuthoringHolder = (smallView = false) => {
            const isCollapsed = this.collapsed === 'authoring';
            this._renderingAuthor = true;
            return (h("div", { class: classnames('authoring-holder', {
                    collapsed: this.collapsed === 'authoring',
                    toggled: this.isToggled()
                }) },
                h("div", { class: classnames('control-bar', {
                        justElement: this.justElement
                    }) }, this.renderAuthoringHeader(smallView)),
                isCollapsed &&
                    this.renderCollapsedPanel('Authoring View', this.isToggled()),
                !isCollapsed && (h("div", { ref: el => el && (this.elementParent1 = el), class: classnames('element-holder', {
                        justElement: this.justElement
                    }) },
                    h("div", { class: "element-parent", style: {
                            minHeight: `${this.minHeightAuthoring}px`
                        } },
                        h("pie-author", { class: "pie-author", ref: el => el && (this.pieAuthor = el), config: this.config, configSettings: this.configureObject, canWatchConfigSettings: true, reFetchBundle: true }, ' ')))),
                h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r) })));
        };
        this.renderStudentHolder = (smallView = false) => {
            const isCollapsed = this.collapsed === 'student';
            this._renderingStudent = true;
            if (!this._renderStudent) {
                return null;
            }
            return (h("div", { class: classnames('student-view-holder', {
                    collapsed: this.collapsed === 'student',
                    toggled: this.isToggled()
                }) },
                h("div", { class: classnames('control-bar', {
                        justElement: this.justElement
                    }) }, this.renderStudentHeader(smallView)),
                isCollapsed && this.renderCollapsedPanel('Student View'),
                !isCollapsed && (h("div", { class: classnames('element-holder', {
                        toggled: this.studSettVisible,
                        justElement: this.justElement
                    }) },
                    h("div", { class: classnames('score-holder', { visible: this.env['mode'] === 'evaluate' }) },
                        h("span", null, "Scores:"),
                        Object.keys(this.scores).map((scoreKey, index) => (h("div", { class: "score" },
                            h("span", null,
                                "Item ",
                                index + 1,
                                ": "),
                            " ",
                            this.scores[scoreKey].score,
                            this.scores[scoreKey].json && h("pre", null, this.scores[scoreKey].json))))),
                    h("div", { ref: el => el && (this.elementParent2 = el), class: "element-parent", style: {
                            minHeight: `${this.minHeightStudent}px`
                        } },
                        h("pie-player", { class: "pie-player", ref: el => el && (this.piePlayer = el), config: this.config, reFetchBundle: true }, ' '))))));
        };
        this.renderContent = () => {
            if (this.rootElWidth >= 1200) {
                return (h("div", { class: "config-holder" },
                    this.renderAuthoringHolder(),
                    h("span", { class: classnames('divider', {
                            larger: this.studSettVisible && this.authSettVisible
                        }) }),
                    this.renderStudentHolder()));
            }
            return (h("div", { class: classnames('smaller-view', {
                    mobile: this.rootElWidth <= 700
                }) },
                this.mobileTabIndex === 0 && this.renderAuthoringHolder(true),
                this.mobileTabIndex === 1 && this.renderStudentHolder(true)));
        };
        this.renderDocHolder = () => {
            return (h("div", { class: classnames('doc-holder', {
                    visible: this.docHolderVisible,
                    mobile: this.rootElWidth <= 700
                }) },
                h("span", { class: "close-icon", onClick: () => (this.docHolderVisible = false) }, "X"),
                h("div", { id: "schema-holder" })));
        };
        this.handleFileInputChange = (e) => {
            const input = e.target;
            if (!this.imageHandler) {
                console.error('no image handler - but file input change triggered?');
                return;
            }
            log('[handleInputFiles] input.files: ', input);
            const files = input.files;
            if (files.length < 1 || !files[0]) {
                log('[handleInputFiles] = !!! no files.');
                this.imageHandler.cancel();
                this.imageHandler = null;
            }
            else {
                const file = files[0];
                this.imageHandler.fileChosen(file);
                this.fileInput.value = '';
                const reader = new FileReader();
                reader.onload = () => {
                    log('[reader.onload]');
                    const dataURL = reader.result;
                    setTimeout(() => {
                        this.imageHandler.done(null, dataURL.toString());
                        this.imageHandler = null;
                    }, 2000);
                };
                log('call readAsDataUrl...', file);
                let progress = 0;
                this.imageHandler.progress(progress, 0, 100);
                range(1, 100).forEach(n => {
                    setTimeout(() => {
                        this.imageHandler.progress(n, n, 100);
                    }, n * 20);
                });
                reader.readAsDataURL(file);
            }
        };
        this.handleRootEl = this.handleRootEl.bind(this);
        this.handleInsertImage = (e) => {
            log('[handleInsertImage] fileInput:', this.fileInput);
            this.imageHandler = e.detail;
            this.fileInput.click();
        };
        this.handleDeleteImage = (e) => {
            e.detail.done();
        };
    }
    componentDidUpdate() {
        if (this.config && 'elements' in this.config && this.config.elements) {
            const elements = this.config.elements;
            Object.keys(elements).map(element => {
                try {
                    const authorElName = `${element}-config`;
                    const authorElement = document.querySelector(authorElName);
                    const configKey = getPackageWithoutVersion(elements[element]);
                    if (authorElement) {
                        // @ts-ignore
                        this.configureObject[configKey] = authorElement._configuration;
                    }
                }
                catch (e) {
                    console.log(e.toString());
                }
            });
        }
        if (this.fileInput) {
            this.fileInput.addEventListener('change', this.handleFileInputChange);
        }
        const isAuthoringToggled = this.isToggled('authoring');
        if (isAuthoringToggled) {
            let offset = 0;
            if (this.bottomContentRef) {
                const ref = this.bottomContentRef.querySelector('.tabs');
                if (ref) {
                    const clientRect = ref.getBoundingClientRect();
                    offset = clientRect.y;
                    this.bottomContentRef.style.height = `calc(100vh - ${offset}px)`;
                }
            }
        }
    }
    collapsePanel(name) {
        this.collapsed = this.collapsed === name ? null : name;
    }
    toggleAuthoringSettings() {
        this.authSettVisible = !this.authSettVisible;
    }
    async validateItem() {
        // Just call the validation method from pie-author, it will do everything that needs to be done
        // @ts-ignore
        await this.pieAuthor.validateModels();
    }
    toggleStudentSettings() {
        this.studSettVisible = !this.studSettVisible;
    }
    isToggled(type = 'student') {
        if (type === 'student') {
            return this.studSettVisible && this.collapsed !== 'student';
        }
        return this.authSettVisible && this.collapsed !== 'authoring';
    }
    detachListeners() {
        if (this.rootEl) {
            this.rootResizeObserver.unobserve(this.rootEl);
        }
    }
    watchConfig(newConfig) {
        Object.values(newConfig.elements || {}).map((packageNAME) => {
            let packageCopy = packageNAME;
            customElements.whenDefined(`pp-${packageToElementName(packageNAME)}`).then(async () => {
                // TODO - what if same element reloaded, could elems be redefined? may need to undefine prior?
                const packageWithoutVersion = getPackageWithoutVersion(packageCopy);
                // todo for some reason, the previous components are no longer on window.pie.default, not sure why
                if (window['pie'].default[packageWithoutVersion].controller || defaultController) {
                    this.pieControllers[packageWithoutVersion] = window['pie'].default[packageWithoutVersion].controller || defaultController;
                }
                // if (this.model) {
                //     this.updatePieModelUsingController(this.model, this.session, this.env);
                // }
                this.state = ViewState.READY;
            });
            if (this.load) {
                this.state = ViewState.READY;
            }
        });
    }
    async updatePieModelUsingController(model, env, shouldResetSession) {
        if ('elements' in this.config && 'markup' in this.config) {
            const properController = this.getProperController(model).controller;
            const modelIndex = this.config.models.findIndex(m => m.id === model.id);
            const session = this.sessions[model.id];
            if (properController && properController.model) {
                const newConfig = await properController.model(model, session, env);
                if (this.env['mode'] === 'evaluate') {
                    const scoring = await properController.outcome(this.config.models.find(m => m.id === model.id), session, this.env);
                    this.scores[model.id] = {
                        score: scoring.score,
                        json: jsonBeautify(scoring.details, null, 2, 80)
                    };
                }
                if (this.piePlayer) {
                    //@ts-ignore
                    const newModel = merge(cloneDeep(this.config.models.find(m => m.id === model.id), newConfig));
                    const config = cloneDeep(this.config);
                    config.models[modelIndex] = newModel;
                    if (shouldResetSession && this.piePlayer.session) {
                        // only reset that model's session
                        this.piePlayer.session.data[this.piePlayer.session.data.findIndex(s => s.id === model.id)] = {};
                    }
                    this.piePlayer.config = config;
                }
            }
        }
    }
    handleRootEl(current) {
        if (current) {
            this.rootEl = current;
            this.rootResizeObserver.observe(current, {
                attributes: true,
                childList: true,
                subtree: true
            });
        }
        else if (this.rootEl) {
            this.rootResizeObserver.unobserve(this.rootEl);
        }
    }
    componentWillLoad() {
        log('component will load ... ');
        this.watchConfig(this.config);
        const rootHandler = throttle(() => {
            if (this.rootEl) {
                this.rootElWidth = this.rootEl.offsetWidth;
            }
        }, 1000, { leading: true });
        this.rootResizeObserver = new ResizeObserver(rootHandler);
        if (this.configSettings) {
            this.configureObject = cloneDeep(this.configSettings);
        }
        if (this.modelSchemaJSONURI) {
            this.watchModelSchemaJSONURI(this.modelSchemaJSONURI);
        }
        if (this.configureSchemaJSONURI) {
            this.watchConfigureSchemaJSONURI(this.configureSchemaJSONURI);
        }
        window.global = window;
        docson.templateBaseUrl = '/assets/html';
    }
    /**
     * Pick up @pie-framework/pie-configure-events and trigger an update.
     * TODO: Why cant i use ModelUpdatedEvent.TYPE in the decorator?
     */
    onModelUpdated(event) {
        if (this._renderingAuthor || this._renderingStudent) {
            this._renderingAuthor = false;
            this._renderingStudent = false;
            return;
        }
        if (!isEqual(this.pieAuthor.config, this.config)) {
            this.config = this.pieAuthor.config;
        }
        log('MODEL UPDATED: target:', event.target, event.detail);
        this.updatePieModelUsingController(event.detail.update, this.env, true);
    }
    onModelLoaded() {
        if (!this._renderStudent) {
            this._renderStudent = true;
        }
    }
    onSessionChanged(event) {
        log('MODEL UPDATED: target:', event.target, event.detail);
        if (event.target) {
            this.sessions[event.target.session.id] = event.target.session;
        }
    }
    watchModelSchemaJSONURI(newUrl) {
        if (newUrl) {
            fetch(newUrl, {
                mode: 'cors'
            })
                .then((response) => response.json())
                .then(response => {
                this.modelSchemaJSON = response;
            });
        }
    }
    watchConfigureSchemaJSONURI(newUrl) {
        if (newUrl) {
            fetch(newUrl, {
                mode: 'cors'
            })
                .then((response) => response.json())
                .then(response => {
                this.configureSchemaJSON = response;
            });
        }
    }
    setMode(mode) {
        this.env = Object.assign(Object.assign({}, this.env), { mode: mode });
        if (this.piePlayer) {
            this.piePlayer.env = { mode: mode, role: this.currentOption };
        }
        if ('models' in this.config) {
            this.config.models.forEach(model => {
                this.updatePieModelUsingController(model, this.env);
            });
        }
    }
    setOption(option) {
        this.currentOption = option;
        if (this.piePlayer) {
            this.piePlayer.env = { mode: this.env['mode'], role: this.currentOption };
        }
    }
    customCheckBox({ label, checked, value, action = undefined }) {
        return (h("label", { class: "custom-checkbox", onClick: () => action.call(this, value) },
            h("i", { class: "material-icons" }, checked ? 'radio_button_checked' : 'radio_button_unchecked'),
            h("span", null, label)));
    }
    renderHeaderTitleInfo({ title, description, options = undefined }) {
        return (h("div", { class: "header-title" },
            h("div", { class: "title-info" },
                h("h4", null, title),
                options &&
                    options.map(opt => (h("span", { class: "option" },
                        h("i", { class: "fa fa-circle" }),
                        opt)))),
            h("span", null, description)));
    }
    renderAuthoringBottomContent() {
        return this.renderTabs([
            {
                name: 'Item Config',
                content: () => {
                    this.cachedJsonConfig1 = cloneDeep(this.config);
                    return this.renderJsonConfigPanel(this.cachedJsonConfig1, 1);
                }
            },
            {
                name: 'Authoring View Settings',
                content: () => {
                    return this.renderJsonConfigPanel(this.configureObject, 2);
                }
            }
        ], 200);
    }
    renderAuthoringHeader(smallView = false) {
        const isToggled = this.isToggled('authoring');
        return (h("div", { class: classnames('authoring-header', {
                collapsed: this.collapsed === 'authoring',
                toggled: isToggled
            }) },
            h("div", { class: "topContent" },
                this.renderHeaderTitleInfo({
                    title: isToggled ? 'Dev Options' : 'Authoring View',
                    description: isToggled
                        ? 'Lorem ipsum dolor sit amet, consectetur adipisicing elit.'
                        : 'The view an author sees when configuring this interaction.'
                }),
                !isToggled && (h("div", { class: "buttons-container" },
                    h("div", { class: "toggle-container", onClick: () => this.validateItem() },
                        h("span", { class: "toggle-text" }, "Validate")),
                    h("div", { class: classnames('toggle-container', {
                            toggled: this.isToggled('authoring')
                        }), onClick: () => this.toggleAuthoringSettings() },
                        h("i", { class: "material-icons toggle-icon" }, this.authSettVisible ? 'toggle_on' : 'toggle_off'),
                        h("span", { class: "toggle-text" }, "Dev Options")),
                    smallView && (h("div", { class: "mobile-button", onClick: () => (this.mobileTabIndex = 1) },
                        h("span", null, "STUDENT VIEW"))))),
                !isToggled && !smallView && (h("i", { class: classnames('material-icons', 'collapse-icon', {
                        collapsed: this.collapsed === 'student'
                    }), onClick: () => this.collapsePanel('student') }, this.collapsed === 'student'
                    ? 'format_indent_decrease'
                    : 'format_indent_increase')),
                isToggled && (h("span", { class: "close-icon", onClick: () => this.toggleAuthoringSettings() }, "X"))),
            h("div", { ref: el => (this.bottomContentRef = el), style: {
                    height: '100vh'
                }, class: "bottomContent authoring" }, isToggled && this.renderAuthoringBottomContent())));
    }
    renderRoleConfigContainer() {
        return (h("div", { class: "roles-settings" },
            h("h5", null, "Role"),
            h("div", { class: "roles-options" },
                this.customCheckBox({
                    label: 'Student',
                    checked: this.currentOption === 'student',
                    value: 'student',
                    action: this.setOption
                }),
                this.customCheckBox({
                    label: 'Instructor',
                    checked: this.currentOption === 'instructor',
                    value: 'instructor',
                    action: this.setOption
                }))));
    }
    renderModeConfigContainer() {
        return (h("div", { class: "modes-settings" },
            h("h5", null, "Mode"),
            h("div", { class: "modes-options" },
                this.customCheckBox({
                    label: 'Gather',
                    checked: this.env['mode'] === 'gather',
                    value: 'gather',
                    action: this.setMode
                }),
                this.customCheckBox({
                    label: 'View',
                    checked: this.env['mode'] === 'view',
                    value: 'view',
                    action: this.setMode
                }),
                this.customCheckBox({
                    label: 'Evaluate',
                    checked: this.env['mode'] === 'evaluate',
                    value: 'evaluate',
                    action: this.setMode
                }))));
    }
    renderSettingsContainer() {
        return (h("div", { class: "settings-tab-container" },
            this.renderModeConfigContainer(),
            this.renderRoleConfigContainer()));
    }
    renderBottomContent() {
        return this.renderTabs([
            {
                name: 'Settings',
                content: this.renderSettingsContainer()
            },
            {
                name: 'Embed',
                content: null
            }
        ]);
    }
    renderStudentHeader(smallView = false) {
        return (h("div", { class: classnames('student-view-header', {
                collapsed: this.collapsed === 'student',
                toggled: this.isToggled()
            }) },
            h("div", { class: "topContent" },
                this.renderHeaderTitleInfo({
                    title: 'Student view',
                    description: 'The view a student (or instructor) sees when entering or reviewing the interaction.',
                    options: [this.env['mode'], this.currentOption]
                }),
                h("div", { class: "buttons-container" },
                    h("div", { class: classnames('toggle-container', 'student', {
                            toggled: this.isToggled()
                        }), onClick: () => this.toggleStudentSettings() },
                        h("i", { class: "material-icons toggle-icon" }, this.studSettVisible ? 'toggle_on' : 'toggle_off'),
                        h("span", { class: "toggle-text" }, "Options"))),
                !smallView && (h("i", { class: classnames('material-icons', 'collapse-icon', {
                        collapsed: this.collapsed === 'authoring'
                    }), onClick: () => this.collapsePanel('authoring') }, this.collapsed === 'authoring'
                    ? 'format_indent_increase'
                    : 'format_indent_decrease')),
                smallView && (h("div", { class: "mobile-button", onClick: () => (this.mobileTabIndex = 0) },
                    h("span", null, "AUTHORING VIEW")))),
            h("div", { class: "bottomContent" }, this.renderBottomContent())));
    }
    renderCollapsedPanel(title, toggled = undefined) {
        return (h("div", { class: classnames('collapsed-panel', {
                toggled: toggled
            }) },
            h("span", null, title)));
    }
    render() {
        switch (this.state) {
            case ViewState.LOADING:
                return (h("div", { class: "root loading" },
                    h("div", { class: "lmask" })));
            case ViewState.ERROR:
                return h("div", { id: "error" }, "Error...");
            case ViewState.READY:
                return (h("div", { class: classnames('root', {
                        multiplePies: this.multiplePies
                    }), ref: this.handleRootEl },
                    this.renderDocHolder(),
                    this.renderContent()));
        }
    }
    static get is() { return "pie-demo-content"; }
    static get originalStyleUrls() { return {
        "$": ["pie-demo.css"]
    }; }
    static get styleUrls() { return {
        "$": ["pie-demo.css"]
    }; }
    static get properties() { return {
        "justElement": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Removes control bar if true"
            },
            "attribute": "just-element",
            "reflect": false,
            "defaultValue": "false"
        },
        "multiplePies": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Makes the element have 50% height"
            },
            "attribute": "multiple-pies",
            "reflect": false,
            "defaultValue": "false"
        },
        "modelSchemaJSONURI": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Link to the pie-schema markdown file"
            },
            "attribute": "model-schema-j-s-o-n-u-r-i",
            "reflect": false,
            "defaultValue": "null"
        },
        "configureSchemaJSONURI": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Link to the pie-schema markdown file"
            },
            "attribute": "configure-schema-j-s-o-n-u-r-i",
            "reflect": false,
            "defaultValue": "null"
        },
        "load": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Tells the component if it needs to load the elements or not"
            },
            "attribute": "load",
            "reflect": false,
            "defaultValue": "true"
        },
        "editor": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include an editor in the view"
            },
            "attribute": "editor",
            "reflect": false,
            "defaultValue": "true"
        },
        "preview": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include an item preview in the view"
            },
            "attribute": "preview",
            "reflect": false,
            "defaultValue": "true"
        },
        "playerControls": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include control panel for adjusting player settings."
            },
            "attribute": "player-controls",
            "reflect": false,
            "defaultValue": "true"
        },
        "configSettings": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "{\n    [key: string]: Object;\n  }",
                "resolved": "{ [key: string]: Object; }",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            }
        },
        "config": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "ItemConfig",
                "resolved": "AdvancedItemConfig | PieContent",
                "references": {
                    "ItemConfig": {
                        "location": "import",
                        "path": "@pie-framework/pie-player-components/dist/types/interface"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            }
        }
    }; }
    static get states() { return {
        "state": {},
        "package": {},
        "rootResizeObserver": {},
        "pieControllers": {},
        "configureObject": {},
        "minHeightAuthoring": {},
        "minHeightStudent": {},
        "rootElWidth": {},
        "bottomContElWidth": {},
        "tabIndex": {},
        "mobileTabIndex": {},
        "currentOption": {},
        "collapsed": {},
        "studSettVisible": {},
        "authSettVisible": {},
        "docHolderVisible": {},
        "env": {},
        "sessions": {},
        "scores": {},
        "_renderStudent": {}
    }; }
    static get watchers() { return [{
            "propName": "config",
            "methodName": "watchConfig"
        }, {
            "propName": "modelSchemaJSONURI",
            "methodName": "watchModelSchemaJSONURI"
        }, {
            "propName": "configureSchemaJSONURI",
            "methodName": "watchConfigureSchemaJSONURI"
        }]; }
    static get listeners() { return [{
            "name": "model.updated",
            "method": "onModelUpdated",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "modelLoaded",
            "method": "onModelLoaded",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "session-changed",
            "method": "onSessionChanged",
            "target": undefined,
            "capture": false,
            "passive": false
        }]; }
}
