import { Component, h, Prop } from "@stencil/core";
import "@pie-framework/pie-player-components";
import { getPackageWithoutVersion } from "../../util/utils";
/**
 * This component tries to add multi-items support.
 * It uses many parts from the old component (pie-demo) but also has some pretty big changes
 * Instead of removing the old component, we'll keep it in case the new component
 * breaks, to be able to switch quickly between them
 * The long-term purpose is to support multi-items, I guess, but for now (the reason I started this)
 *    teh purpose was to be able to test complex-rubric integrations
 * The documentation for each item is still item-type oriented now (not multi-item)
 *    ex: math-inline in pie-website will only have math-inline documentation, even if it has multiple item types
**/
export class PieDemoNew {
    constructor() {
        /**
         * Removes control bar if true
         * @type {boolean}
         */
        this.justElement = false;
        /**
         * Makes the element have 50% height
         * @type {boolean}
         */
        this.multiplePies = false;
        /**
         * Link to the pie-schema markdown file
         * @type {boolean}
         */
        this.modelSchemaJSONURI = null;
        /**
         * Link to the pie-schema markdown file
         * @type {boolean}
         */
        this.configureSchemaJSONURI = null;
        /**
         * Tells the component if it needs to load the elements or not
         */
        this.load = true;
        /**
         * Include an editor in the view
         */
        this.editor = true;
        /**
         * Include an item preview in the view
         */
        this.preview = true;
        /**
         * Include control panel for adjusting player settings.
         */
        this.playerControls = true;
    }
    render() {
        let pieName = this.pie
            .substr(this.pie.lastIndexOf("/") + 1, this.pie.length)
            .split("@")[0];
        if (pieName.indexOf("-") < 0) {
            pieName = `x-${pieName}`;
        }
        const packageWithoutVersion = getPackageWithoutVersion(this.pie);
        return h("pie-demo-content", { justElement: this.justElement, multiplePies: this.multiplePies, modelSchemaJSONURI: this.modelSchemaJSONURI, configureSchemaJSONURI: this.configureSchemaJSONURI, load: this.load, editor: this.editor, preview: this.preview, playerControls: this.playerControls, config: {
                id: "1",
                elements: {
                    [pieName]: this.pie
                },
                models: [
                    this.model
                ],
                markup: `<${pieName} id='1'></${pieName}>`
            }, configSettings: {
                [packageWithoutVersion]: this.configure
            } });
    }
    static get is() { return "pie-demo"; }
    static get originalStyleUrls() { return {
        "$": ["pie-demo.css"]
    }; }
    static get styleUrls() { return {
        "$": ["pie-demo.css"]
    }; }
    static get properties() { return {
        "pie": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The PIE npm package to demo. e.g. `@pie-element/multiple-choice`"
            },
            "attribute": "pie",
            "reflect": false
        },
        "justElement": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Removes control bar if true"
            },
            "attribute": "just-element",
            "reflect": false,
            "defaultValue": "false"
        },
        "multiplePies": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Makes the element have 50% height"
            },
            "attribute": "multiple-pies",
            "reflect": false,
            "defaultValue": "false"
        },
        "modelSchemaJSONURI": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Link to the pie-schema markdown file"
            },
            "attribute": "model-schema-j-s-o-n-u-r-i",
            "reflect": false,
            "defaultValue": "null"
        },
        "configureSchemaJSONURI": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Link to the pie-schema markdown file"
            },
            "attribute": "configure-schema-j-s-o-n-u-r-i",
            "reflect": false,
            "defaultValue": "null"
        },
        "load": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Tells the component if it needs to load the elements or not"
            },
            "attribute": "load",
            "reflect": false,
            "defaultValue": "true"
        },
        "editor": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include an editor in the view"
            },
            "attribute": "editor",
            "reflect": false,
            "defaultValue": "true"
        },
        "preview": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include an item preview in the view"
            },
            "attribute": "preview",
            "reflect": false,
            "defaultValue": "true"
        },
        "playerControls": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include control panel for adjusting player settings."
            },
            "attribute": "player-controls",
            "reflect": false,
            "defaultValue": "true"
        },
        "model": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "PieModel",
                "resolved": "PieModel",
                "references": {
                    "PieModel": {
                        "location": "import",
                        "path": "@pie-framework/pie-player-components/dist/types/interface"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The model for the pie."
            }
        },
        "configure": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Object",
                "resolved": "Object",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            }
        }
    }; }
}
