import { Component, h, Prop, Watch, State, Listen } from "@stencil/core";
import "@pie-framework/pie-player-components";
import ResizeObserver from "resize-observer-polyfill";
import jsonBeautify from "json-beautify";
import { getPackageWithoutVersion, packageToElementName } from "../../util/utils";
import classnames from "classnames";
import docson from "docson";
import range from "lodash/range";
import cloneDeep from "lodash/cloneDeep";
import merge from 'lodash/merge';
import throttle from 'lodash/throttle';
import { InsertImageEvent, DeleteImageEvent } from "@pie-framework/pie-configure-events";
import debug from "debug";
const log = debug("pie-framework:pie-demo");
var ViewState;
(function (ViewState) {
    ViewState[ViewState["LOADING"] = 0] = "LOADING";
    ViewState[ViewState["READY"] = 1] = "READY";
    ViewState[ViewState["ERROR"] = 2] = "ERROR";
})(ViewState || (ViewState = {}));
const defaultController = {
    model: (model, session, env) => {
        log(model, session, env);
        return Object.assign(Object.assign({}, model), { mode: env.mode });
    },
    outcome: () => ({
        score: 0,
        details: {}
    })
};
export class PieDemo {
    constructor() {
        /**
         * Removes control bar if true
         * @type {boolean}
         */
        this.justElement = false;
        /**
         * Makes the element have 50% height
         * @type {boolean}
         */
        this.multiplePies = false;
        /**
         * Link to the pie-schema markdown file
         * @type {boolean}
         */
        this.modelSchemaJSONURI = null;
        /**
         * Link to the pie-schema markdown file
         * @type {boolean}
         */
        this.configureSchemaJSONURI = null;
        /**
         * Tells the component if it needs to load the elements or not
         */
        this.load = true;
        /**
         * Include an editor in the view
         */
        this.editor = true;
        /**
         * Include an item preview in the view
         */
        this.preview = true;
        /**
         * Include control panel for adjusting player settings.
         */
        this.playerControls = true;
        this.state = ViewState.LOADING;
        this.minHeightAuthoring = "initial";
        this.minHeightStudent = "initial";
        this.rootElWidth = 1250;
        this.bottomContElWidth = 550;
        this.tabIndex = 0;
        this.mobileTabIndex = 0;
        this.currentOption = "student";
        this.studSettVisible = false;
        this.authSettVisible = false;
        this.docHolderVisible = false;
        this.env = { mode: "gather" };
        this.session = {};
        this.scoring = {
            score: 0,
            json: ""
        };
        this.jsonConfigValue = null;
        this._renderStudent = false;
        this.imageHandler = null;
        this.configureSchemaJSON = null;
        this.modelSchemaJSON = null;
        this.cachedJsonConfig1 = null;
        this.cachedJsonConfig2 = null;
        this.rootEl = null;
        this.fileInput = null;
        this._renderingAuthor = false;
        this._renderingStudent = false;
        this.viewDocumentation = tabIndex => {
            log("Tab Index", tabIndex);
            const currentContent = tabIndex === 1 ? this.modelSchemaJSON : this.configureSchemaJSON;
            this.docHolderVisible = true;
            docson.doc("schema-holder", currentContent);
        };
        this.renderJsonConfigPanel = (jsonData, index) => {
            log(jsonData);
            return (h("div", { class: "json-config" },
                h("div", { class: "json-config-header" },
                    h("div", { class: "view-container", onClick: () => this.viewDocumentation(index) },
                        h("i", { class: "fa fa-desktop" }),
                        h("span", null, "View Documentation")),
                    h("div", { class: "right-content" },
                        h("div", { class: "copy-container", onClick: () => {
                                this[`textAreaContentRef${index}`].select();
                                document.execCommand("copy");
                            } },
                            h("i", { class: "fa fa-clone" }),
                            h("span", null, "Copy JSON")),
                        h("div", { class: "reset-container", onClick: () => {
                                if (this[`textAreaContentRef${index}`]) {
                                    this[`textAreaContentRef${index}`].value = jsonBeautify(this[`cachedJsonConfig${index}`], null, 2, 98);
                                    if (index === 1) {
                                        this.configModel = this[`cachedJsonConfig${index}`];
                                    }
                                    else {
                                        this.configureObject = this[`cachedJsonConfig${index}`];
                                    }
                                }
                            } },
                            h("i", { class: "fa fa-undo" }),
                            h("span", null, "Reset")))),
                h("textarea", { ref: el => (this[`textAreaContentRef${index}`] = el), class: "json-content", value: jsonBeautify(jsonData, null, 2, 98), onChange: e => {
                        const target = e.target;
                        if (index === 1) {
                            this.configModel = JSON.parse(target.value);
                        }
                        else {
                            this.configureObject = JSON.parse(target.value);
                        }
                    } })));
        };
        this.renderTabs = (tabs, tabWidth = 100) => {
            const content = tabs[this.tabIndex] ? tabs[this.tabIndex].content : null;
            const contentValue = typeof content === "function" ? content() : content;
            return (h("div", { class: "tabs-container" },
                h("div", { class: "tabs" }, tabs.map((tab, index) => (h("div", { style: {
                        width: `${tabWidth}px`
                    }, class: classnames("tab", {
                        selected: this.tabIndex === index
                    }), onClick: () => (this.tabIndex = index) }, tab.name)))),
                h("span", { class: "selected-line", style: {
                        left: `${this.tabIndex * tabWidth}px`,
                        width: `${tabWidth}px`
                    } }),
                h("div", { class: "tab-content" }, contentValue)));
        };
        this.renderAuthoringHolder = (smallView = false) => {
            const ConfigTag = this.pieName + "-config";
            const isCollapsed = this.collapsed === "authoring";
            const config = {
                id: "1",
                elements: {
                    [ConfigTag]: this.pie
                },
                models: [
                    this.configModel
                ],
                markup: `
          <${ConfigTag} id='1'></${ConfigTag}>
        `
            };
            const configSettings = {
                [getPackageWithoutVersion(this.pie)]: this.configureObject
            };
            this._renderingAuthor = true;
            return (h("div", { class: classnames("authoring-holder", {
                    collapsed: this.collapsed === "authoring",
                    toggled: this.isToggled()
                }) },
                h("div", { class: classnames("control-bar", {
                        justElement: this.justElement
                    }) }, this.renderAuthoringHeader(smallView)),
                isCollapsed &&
                    this.renderCollapsedPanel("Authoring View", this.isToggled()),
                !isCollapsed && (h("div", { ref: el => el && (this.elementParent1 = el), class: classnames("element-holder", {
                        justElement: this.justElement
                    }) },
                    h("div", { class: "element-parent", style: {
                            minHeight: `${this.minHeightAuthoring}px`
                        } },
                        h("pie-author", { class: "pie-author", ref: el => el && (this.pieAuthor = el), config: config, configSettings: configSettings, reFetchBundle: true }, " ")))),
                h("input", { type: "file", hidden: true, ref: r => (this.fileInput = r) })));
        };
        this.renderStudentHolder = (smallView = false) => {
            const TagName = this.pieName + "";
            const isCollapsed = this.collapsed === "student";
            const config = {
                id: "1",
                elements: {
                    [TagName]: this.pie
                },
                models: [
                    this.configModel
                ],
                markup: `
          <${TagName} id='1'></${TagName}>
        `
            };
            this._renderingStudent = true;
            if (!this._renderStudent) {
                return null;
            }
            return (h("div", { class: classnames("student-view-holder", {
                    collapsed: this.collapsed === "student",
                    toggled: this.isToggled()
                }) },
                h("div", { class: classnames("control-bar", {
                        justElement: this.justElement
                    }) }, this.renderStudentHeader(smallView)),
                isCollapsed && this.renderCollapsedPanel("Student View"),
                !isCollapsed && (h("div", { class: classnames("element-holder", {
                        toggled: this.studSettVisible,
                        justElement: this.justElement
                    }) },
                    h("div", { class: classnames("score-holder", {
                            visible: this.env["mode"] === "evaluate"
                        }) },
                        h("div", { class: "score" },
                            h("span", null, "Score: "),
                            " ",
                            this.scoring.score),
                        h("pre", null, this.scoring.json)),
                    h("div", { ref: el => el && (this.elementParent2 = el), class: "element-parent", style: {
                            minHeight: `${this.minHeightStudent}px`
                        } },
                        h("pie-player", { class: "pie-player", ref: el => el && (this.piePlayer = el), config: config, reFetchBundle: true }, " "))))));
        };
        this.renderContent = () => {
            if (this.rootElWidth >= 1200) {
                return (h("div", { class: "config-holder" },
                    this.renderAuthoringHolder(),
                    h("span", { class: classnames("divider", {
                            larger: this.studSettVisible && this.authSettVisible
                        }) }),
                    this.renderStudentHolder()));
            }
            return (h("div", { class: classnames("smaller-view", {
                    mobile: this.rootElWidth <= 700
                }) },
                this.mobileTabIndex === 0 && this.renderAuthoringHolder(true),
                this.mobileTabIndex === 1 && this.renderStudentHolder(true)));
        };
        this.renderDocHolder = () => {
            return (h("div", { class: classnames("doc-holder", {
                    visible: this.docHolderVisible,
                    mobile: this.rootElWidth <= 700
                }) },
                h("span", { class: "close-icon", onClick: () => (this.docHolderVisible = false) }, "X"),
                h("div", { id: "schema-holder" })));
        };
        this.handleFileInputChange = (e) => {
            const input = e.target;
            if (!this.imageHandler) {
                console.error("no image handler - but file input change triggered?");
                return;
            }
            log("[handleInputFiles] input.files: ", input);
            const files = input.files;
            if (files.length < 1 || !files[0]) {
                log("[handleInputFiles] = !!! no files.");
                this.imageHandler.cancel();
                this.imageHandler = null;
            }
            else {
                const file = files[0];
                this.imageHandler.fileChosen(file);
                this.fileInput.value = "";
                const reader = new FileReader();
                reader.onload = () => {
                    log("[reader.onload]");
                    const dataURL = reader.result;
                    setTimeout(() => {
                        this.imageHandler.done(null, dataURL.toString());
                        this.imageHandler = null;
                    }, 2000);
                };
                log("call readAsDataUrl...", file);
                let progress = 0;
                this.imageHandler.progress(progress, 0, 100);
                range(1, 100).forEach(n => {
                    setTimeout(() => {
                        this.imageHandler.progress(n, n, 100);
                    }, n * 20);
                });
                reader.readAsDataURL(file);
            }
        };
        this.handleRootEl = this.handleRootEl.bind(this);
        this.handleInsertImage = (e) => {
            log("[handleInsertImage] fileInput:", this.fileInput);
            this.imageHandler = e.detail;
            this.fileInput.click();
        };
        this.handleDeleteImage = (e) => {
            e.detail.done();
        };
    }
    componentDidUpdate() {
        if (this.pieAuthor) {
            const authorElName = `${Object.keys(this.pieAuthor.config.elements)[0]}-config`;
            if (authorElName) {
                let findEl;
                if (!this.authorElement) {
                    findEl = true;
                }
                else {
                    const tagName = this.authorElement.tagName.toLowerCase();
                    findEl = tagName !== authorElName;
                }
                if (findEl) {
                    this.authorElement = document.querySelector(authorElName);
                }
            }
        }
        if (this.fileInput) {
            this.fileInput.addEventListener("change", this.handleFileInputChange);
        }
        const isAuthoringToggled = this.isToggled("authoring");
        if (isAuthoringToggled) {
            let offset = 0;
            if (this.bottomContentRef) {
                const ref = this.bottomContentRef.querySelector(".tabs");
                if (ref) {
                    const clientRect = ref.getBoundingClientRect();
                    offset = clientRect.y;
                    this.bottomContentRef.style.height = `calc(100vh - ${offset}px)`;
                }
            }
        }
    }
    collapsePanel(name) {
        this.collapsed = this.collapsed === name ? null : name;
    }
    toggleAuthoringSettings() {
        this.authSettVisible = !this.authSettVisible;
    }
    async validateItem() {
        if (this.pieController && this.pieController.model) {
            try {
                const errors = await this.pieController.validate(this.configModel, this.configure);
                console.warn('errors = ', errors);
                this.updateModel(Object.assign(Object.assign({}, this.configModel), { errors }));
            }
            catch (e) {
                console.error(e.toString());
            }
        }
    }
    toggleStudentSettings() {
        this.studSettVisible = !this.studSettVisible;
    }
    isToggled(type = "student") {
        if (type === "student") {
            return this.studSettVisible && this.collapsed !== "student";
        }
        return this.authSettVisible && this.collapsed !== "authoring";
    }
    detachListeners() {
        if (this.configElement) {
            this.configElement.removeEventListener("model.updated", this.handleSetConfigElement);
        }
        if (this.rootEl) {
            this.rootResizeObserver.unobserve(this.rootEl);
        }
    }
    watchPie(newPie) {
        log("[watchPie] ", newPie);
        this.package = newPie;
        this.pieName = newPie
            .substr(newPie.lastIndexOf("/") + 1, newPie.length)
            .split("@")[0];
        if (this.pieName.indexOf("-") < 0) {
            this.pieName = `x-${this.pieName}`;
        }
        customElements.whenDefined(`pp-${packageToElementName(this.pie)}`).then(async () => {
            // TODO - what if same element reloaded, could elems be redefined? may need to undefine prior?
            const packageWithoutVersion = getPackageWithoutVersion(this.package);
            this.pieController =
                (window["pie"].default[packageWithoutVersion] && window["pie"].default[packageWithoutVersion].controller) ||
                    defaultController;
            if (this.model) {
                this.updatePieModelFromController(this.model, this.session, this.env);
            }
            this.state = ViewState.READY;
        });
        if (this.load) {
            this.state = ViewState.READY;
        }
    }
    updateModel(newModel) {
        this.configModel = newModel;
    }
    async updateConfigure(newConfigure) {
        this.configureObject = newConfigure;
    }
    watchConfigModel(newModel) {
        if (this.configElement)
            this.configElement.model = newModel;
        this.updatePieModelFromController(newModel, this.session, this.env, true);
    }
    async updatePieModelFromController(model, session, env, shouldResetSession) {
        if (this.pieController && this.pieController.model) {
            const newConfig = await this.pieController.model(model, session, env);
            if (this.env["mode"] === "evaluate") {
                const scoring = await this.pieController.outcome(this.configModel, this.session, this.env);
                this.scoring = {
                    score: scoring.score,
                    json: jsonBeautify(scoring.details, null, 2, 80)
                };
            }
            if (this.piePlayer) {
                const ConfigTag = this.pieName + "-config";
                const newModel = merge(cloneDeep(this.configModel, newConfig));
                const config = {
                    id: "1",
                    elements: {
                        [ConfigTag]: this.pie
                    },
                    models: [
                        newModel
                    ],
                    markup: `
          <${ConfigTag} id='1'></${ConfigTag}>
        `
                };
                // if the model or config changes, we need to reset session (to avoid the possibility of having a non-valid session)
                if (shouldResetSession) {
                    this.piePlayer.session.data = [];
                }
                this.piePlayer.config = config;
            }
        }
    }
    watchPieElement(pieElement) {
        if (pieElement && !pieElement.model) {
            pieElement.model = this.pieElementModel;
        }
    }
    watchPieElementModel(newModel) {
        if (this.pieElement) {
            this.pieElement.model = newModel;
            this.pieElement.configuration = this.configureObject;
        }
    }
    handleRootEl(current) {
        if (current) {
            this.rootEl = current;
            this.rootResizeObserver.observe(current, {
                attributes: true,
                childList: true,
                subtree: true
            });
        }
        else if (this.rootEl) {
            this.rootResizeObserver.unobserve(this.rootEl);
        }
    }
    componentWillLoad() {
        log("component will load ... ");
        this.watchPie(this.pie);
        const rootHandler = throttle(() => {
            if (this.rootEl) {
                this.rootElWidth = this.rootEl.offsetWidth;
            }
        }, 1000, { leading: true });
        this.rootResizeObserver = new ResizeObserver(rootHandler);
        if (this.model) {
            this.updateModel(this.model);
        }
        if (this.configure) {
            this.updateConfigure(this.configure);
        }
        if (this.modelSchemaJSONURI) {
            this.watchModelSchemaJSONURI(this.modelSchemaJSONURI);
        }
        if (this.configureSchemaJSONURI) {
            this.watchConfigureSchemaJSONURI(this.configureSchemaJSONURI);
        }
        window.global = window;
        docson.templateBaseUrl = "/assets/html";
    }
    handleElementResize(el, name) {
        let minHeight = "initial";
        const navigateNode = el => {
            if (el.nodeType === 1) {
                const allStyle = getComputedStyle(el);
                if (!el.matches(".element-holder") &&
                    allStyle.position === "absolute") {
                    const bounding = el.getBoundingClientRect();
                    const currentHeight = bounding.top + bounding.height;
                    if (minHeight === "initial" || currentHeight > minHeight) {
                        minHeight = currentHeight;
                    }
                }
                if (el.childNodes.length > 0) {
                    el.childNodes.forEach(ch => navigateNode(ch));
                }
            }
        };
        navigateNode(el);
        this[name] = minHeight;
    }
    /**
     * Pick up @pie-framework/pie-configure-events and trigger an update.
     * TODO: Why cant i use ModelUpdatedEvent.TYPE in the decorator?
     */
    onModelUpdated(event) {
        if (this._renderingAuthor || this._renderingStudent) {
            this._renderingAuthor = false;
            this._renderingStudent = false;
            return;
        }
        log("MODEL UPDATED: target:", event.target, event.detail);
        // this.model = event.detail.update;
        this.updatePieModelFromController(event.detail.update, this.session, this.env, true);
    }
    onModelLoaded() {
        if (!this._renderStudent) {
            this._renderStudent = true;
        }
    }
    onSessionChanged(event) {
        log("MODEL UPDATED: target:", event.target, event.detail);
        if (event.target) {
            this.session = event.target.session;
        }
    }
    watchConfigElement(el) {
        if (!el) {
            return;
        }
        el.addEventListener(InsertImageEvent.TYPE, this.handleInsertImage);
        el.addEventListener(DeleteImageEvent.TYPE, this.handleDeleteImage);
    }
    watchModelSchemaJSONURI(newUrl) {
        if (newUrl) {
            fetch(newUrl, {
                mode: "cors"
            })
                .then((response) => response.json())
                .then(response => {
                this.modelSchemaJSON = response;
            });
        }
    }
    watchConfigureSchemaJSONURI(newUrl) {
        if (newUrl) {
            fetch(newUrl, {
                mode: "cors"
            })
                .then((response) => response.json())
                .then(response => {
                this.configureSchemaJSON = response;
            });
        }
    }
    setMode(mode) {
        this.env = Object.assign(Object.assign({}, this.env), { mode: mode });
        if (this.piePlayer) {
            this.piePlayer.env = { mode: mode, role: this.currentOption };
        }
        this.updatePieModelFromController(this.configModel, this.session, this.env);
    }
    setOption(option) {
        this.currentOption = option;
        if (this.piePlayer) {
            this.piePlayer.env = { mode: this.env['mode'], role: this.currentOption };
        }
    }
    customCheckBox({ label, checked, value, action = undefined }) {
        return (h("label", { class: "custom-checkbox", onClick: () => action.call(this, value) },
            h("i", { class: "material-icons" }, checked ? "radio_button_checked" : "radio_button_unchecked"),
            h("span", null, label)));
    }
    renderHeaderTitleInfo({ title, description, options = undefined }) {
        return (h("div", { class: "header-title" },
            h("div", { class: "title-info" },
                h("h4", null, title),
                options &&
                    options.map(opt => (h("span", { class: "option" },
                        h("i", { class: "fa fa-circle" }),
                        opt)))),
            h("span", null, description)));
    }
    renderAuthoringBottomContent() {
        return this.renderTabs([
            {
                name: "Item Model",
                content: () => {
                    this.cachedJsonConfig1 = cloneDeep(this.configModel);
                    return this.renderJsonConfigPanel(this.cachedJsonConfig1, 1);
                }
            },
            {
                name: "Authoring View Settings",
                content: () => {
                    const config = this.authorElement ? this.authorElement._configuration : this.configure;
                    this.cachedJsonConfig2 = cloneDeep(config);
                    return this.renderJsonConfigPanel(this.cachedJsonConfig2, 2);
                }
            }
        ], 200);
    }
    renderAuthoringHeader(smallView = false) {
        const isToggled = this.isToggled("authoring");
        return (h("div", { class: classnames("authoring-header", {
                collapsed: this.collapsed === "authoring",
                toggled: isToggled
            }) },
            h("div", { class: "topContent" },
                this.renderHeaderTitleInfo({
                    title: isToggled ? "Dev Options" : "Authoring View",
                    description: isToggled
                        ? "Lorem ipsum dolor sit amet, consectetur adipisicing elit."
                        : "The view an author sees when configuring this interaction."
                }),
                !isToggled && (h("div", { class: "buttons-container" },
                    h("div", { class: "toggle-container", onClick: () => this.validateItem() },
                        h("span", { class: "toggle-text" }, "Validate")),
                    h("div", { class: classnames("toggle-container", {
                            toggled: this.isToggled("authoring")
                        }), onClick: () => this.toggleAuthoringSettings() },
                        h("i", { class: "material-icons toggle-icon" }, this.authSettVisible ? "toggle_on" : "toggle_off"),
                        h("span", { class: "toggle-text" }, "Dev Options")),
                    smallView && (h("div", { class: "mobile-button", onClick: () => (this.mobileTabIndex = 1) },
                        h("span", null, "STUDENT VIEW"))))),
                !isToggled && !smallView && (h("i", { class: classnames("material-icons", "collapse-icon", {
                        collapsed: this.collapsed === "student"
                    }), onClick: () => this.collapsePanel("student") }, this.collapsed === "student"
                    ? "format_indent_decrease"
                    : "format_indent_increase")),
                isToggled && (h("span", { class: "close-icon", onClick: () => this.toggleAuthoringSettings() }, "X"))),
            h("div", { ref: el => (this.bottomContentRef = el), style: {
                    height: '100vh'
                }, class: "bottomContent authoring" }, isToggled && this.renderAuthoringBottomContent())));
    }
    renderRoleConfigContainer() {
        return (h("div", { class: "roles-settings" },
            h("h5", null, "Role"),
            h("div", { class: "roles-options" },
                this.customCheckBox({
                    label: "Student",
                    checked: this.currentOption === "student",
                    value: "student",
                    action: this.setOption
                }),
                this.customCheckBox({
                    label: "Instructor",
                    checked: this.currentOption === "instructor",
                    value: "instructor",
                    action: this.setOption
                }))));
    }
    renderModeConfigContainer() {
        return (h("div", { class: "modes-settings" },
            h("h5", null, "Mode"),
            h("div", { class: "modes-options" },
                this.customCheckBox({
                    label: "Gather",
                    checked: this.env["mode"] === "gather",
                    value: "gather",
                    action: this.setMode
                }),
                this.customCheckBox({
                    label: "View",
                    checked: this.env["mode"] === "view",
                    value: "view",
                    action: this.setMode
                }),
                this.customCheckBox({
                    label: "Evaluate",
                    checked: this.env["mode"] === "evaluate",
                    value: "evaluate",
                    action: this.setMode
                }))));
    }
    renderSettingsContainer() {
        return (h("div", { class: "settings-tab-container" },
            this.renderModeConfigContainer(),
            this.renderRoleConfigContainer()));
    }
    renderBottomContent() {
        return this.renderTabs([
            {
                name: "Settings",
                content: this.renderSettingsContainer()
            },
            {
                name: "Embed",
                content: null
            }
        ]);
    }
    renderStudentHeader(smallView = false) {
        return (h("div", { class: classnames("student-view-header", {
                collapsed: this.collapsed === "student",
                toggled: this.isToggled()
            }) },
            h("div", { class: "topContent" },
                this.renderHeaderTitleInfo({
                    title: "Student view",
                    description: "The view a student (or instructor) sees when entering or reviewing the interaction.",
                    options: [this.env["mode"], this.currentOption]
                }),
                h("div", { class: "buttons-container" },
                    h("div", { class: classnames("toggle-container", "student", {
                            toggled: this.isToggled()
                        }), onClick: () => this.toggleStudentSettings() },
                        h("i", { class: "material-icons toggle-icon" }, this.studSettVisible ? "toggle_on" : "toggle_off"),
                        h("span", { class: "toggle-text" }, "Options"))),
                !smallView && (h("i", { class: classnames("material-icons", "collapse-icon", {
                        collapsed: this.collapsed === "authoring"
                    }), onClick: () => this.collapsePanel("authoring") }, this.collapsed === "authoring"
                    ? "format_indent_increase"
                    : "format_indent_decrease")),
                smallView && (h("div", { class: "mobile-button", onClick: () => (this.mobileTabIndex = 0) },
                    h("span", null, "AUTHORING VIEW")))),
            h("div", { class: "bottomContent" }, this.renderBottomContent())));
    }
    renderCollapsedPanel(title, toggled = undefined) {
        return (h("div", { class: classnames("collapsed-panel", {
                toggled: toggled
            }) },
            h("span", null, title)));
    }
    render() {
        switch (this.state) {
            case ViewState.LOADING:
                return (h("div", { class: "root loading" },
                    h("div", { class: "lmask" })));
            case ViewState.ERROR:
                return h("div", { id: "error" }, "Error...");
            case ViewState.READY:
                return (h("div", { class: classnames("root", {
                        multiplePies: this.multiplePies
                    }), ref: this.handleRootEl },
                    this.renderDocHolder(),
                    this.renderContent()));
        }
    }
    static get is() { return "pie-demo-old"; }
    static get originalStyleUrls() { return {
        "$": ["pie-demo.css"]
    }; }
    static get styleUrls() { return {
        "$": ["pie-demo.css"]
    }; }
    static get properties() { return {
        "pie": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The PIE npm package to demo. e.g. `@pie-element/multiple-choice`"
            },
            "attribute": "pie",
            "reflect": false
        },
        "justElement": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Removes control bar if true"
            },
            "attribute": "just-element",
            "reflect": false,
            "defaultValue": "false"
        },
        "multiplePies": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Makes the element have 50% height"
            },
            "attribute": "multiple-pies",
            "reflect": false,
            "defaultValue": "false"
        },
        "modelSchemaJSONURI": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Link to the pie-schema markdown file"
            },
            "attribute": "model-schema-j-s-o-n-u-r-i",
            "reflect": false,
            "defaultValue": "null"
        },
        "configureSchemaJSONURI": {
            "type": "string",
            "mutable": false,
            "complexType": {
                "original": "string",
                "resolved": "string",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [{
                        "text": "{boolean}",
                        "name": "type"
                    }],
                "text": "Link to the pie-schema markdown file"
            },
            "attribute": "configure-schema-j-s-o-n-u-r-i",
            "reflect": false,
            "defaultValue": "null"
        },
        "load": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Tells the component if it needs to load the elements or not"
            },
            "attribute": "load",
            "reflect": false,
            "defaultValue": "true"
        },
        "editor": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include an editor in the view"
            },
            "attribute": "editor",
            "reflect": false,
            "defaultValue": "true"
        },
        "preview": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include an item preview in the view"
            },
            "attribute": "preview",
            "reflect": false,
            "defaultValue": "true"
        },
        "playerControls": {
            "type": "boolean",
            "mutable": false,
            "complexType": {
                "original": "boolean",
                "resolved": "boolean",
                "references": {}
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "Include control panel for adjusting player settings."
            },
            "attribute": "player-controls",
            "reflect": false,
            "defaultValue": "true"
        },
        "model": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Object",
                "resolved": "Object",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": "The model for the pie."
            }
        },
        "configure": {
            "type": "unknown",
            "mutable": false,
            "complexType": {
                "original": "Object",
                "resolved": "Object",
                "references": {
                    "Object": {
                        "location": "global"
                    }
                }
            },
            "required": false,
            "optional": false,
            "docs": {
                "tags": [],
                "text": ""
            }
        }
    }; }
    static get states() { return {
        "configModel": {},
        "configureObject": {},
        "state": {},
        "package": {},
        "rootResizeObserver": {},
        "pieName": {},
        "pieController": {},
        "pieElement": {},
        "minHeightAuthoring": {},
        "minHeightStudent": {},
        "rootElWidth": {},
        "bottomContElWidth": {},
        "pieElementModel": {},
        "configElement": {},
        "tabIndex": {},
        "mobileTabIndex": {},
        "currentOption": {},
        "collapsed": {},
        "studSettVisible": {},
        "authSettVisible": {},
        "docHolderVisible": {},
        "env": {},
        "session": {},
        "scoring": {},
        "jsonConfigValue": {},
        "_renderStudent": {}
    }; }
    static get watchers() { return [{
            "propName": "pie",
            "methodName": "watchPie"
        }, {
            "propName": "model",
            "methodName": "updateModel"
        }, {
            "propName": "configure",
            "methodName": "updateConfigure"
        }, {
            "propName": "configModel",
            "methodName": "watchConfigModel"
        }, {
            "propName": "pieElement",
            "methodName": "watchPieElement"
        }, {
            "propName": "pieElementModel",
            "methodName": "watchPieElementModel"
        }, {
            "propName": "configElement",
            "methodName": "watchConfigElement"
        }, {
            "propName": "modelSchemaJSONURI",
            "methodName": "watchModelSchemaJSONURI"
        }, {
            "propName": "configureSchemaJSONURI",
            "methodName": "watchConfigureSchemaJSONURI"
        }]; }
    static get listeners() { return [{
            "name": "model.updated",
            "method": "onModelUpdated",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "modelLoaded",
            "method": "onModelLoaded",
            "target": undefined,
            "capture": false,
            "passive": false
        }, {
            "name": "session-changed",
            "method": "onSessionChanged",
            "target": undefined,
            "capture": false,
            "passive": false
        }]; }
}
