import "@pie-framework/pie-player-components";
import { PieModel } from "@pie-framework/pie-player-components/dist/types/interface";
import { JSX } from "@pie-framework/pie-player-components/dist/types/components";
import { ModelUpdatedEvent, InsertImageEvent, DeleteImageEvent, ImageHandler } from "@pie-framework/pie-configure-events";
declare enum ViewState {
    LOADING = 0,
    READY = 1,
    ERROR = 2
}
interface ScoringObject extends Object {
    score: number;
    details: Object;
}
interface DisplayedScore extends Object {
    score: number;
    json: string;
}
declare type PieController = {
    model: (config: Object, session: Object, env: Object) => Promise<Object>;
    outcome: (config: Object, session: Object, env: Object) => Promise<ScoringObject>;
    validate?: (model: Object, config: Object) => Promise<Object>;
};
interface PieElement extends HTMLElement {
    _model: Object;
    model: Object;
    configuration: Object;
    _configuration: Object;
    session: Object;
    onModelChanged: Function;
}
export declare class PieDemo {
    /**
     * The PIE npm package to demo. e.g. `@pie-element/multiple-choice`
     */
    pie: string;
    /**
     * Removes control bar if true
     * @type {boolean}
     */
    justElement: boolean;
    /**
     * Makes the element have 50% height
     * @type {boolean}
     */
    multiplePies: boolean;
    /**
     * Link to the pie-schema markdown file
     * @type {boolean}
     */
    modelSchemaJSONURI: string;
    /**
     * Link to the pie-schema markdown file
     * @type {boolean}
     */
    configureSchemaJSONURI: string;
    /**
     * Tells the component if it needs to load the elements or not
     */
    load: boolean;
    /**
     * Include an editor in the view
     */
    editor: boolean;
    /**
     * Include an item preview in the view
     */
    preview: boolean;
    /**
     * Include control panel for adjusting player settings.
     */
    playerControls: boolean;
    /**
     * The model for the pie.
     */
    model: Object;
    configure: Object;
    configModel: PieModel;
    configureObject: Object;
    state: ViewState;
    package: string;
    rootResizeObserver: any;
    pieName: string;
    pieController: PieController;
    pieElement: PieElement;
    minHeightAuthoring: any;
    minHeightStudent: any;
    rootElWidth: number;
    bottomContElWidth: number;
    pieElementModel: PieModel;
    configElement: PieElement;
    tabIndex: number;
    mobileTabIndex: number;
    currentOption: string;
    collapsed: string;
    studSettVisible: boolean;
    authSettVisible: boolean;
    docHolderVisible: boolean;
    env: Object;
    session: Object;
    scoring: DisplayedScore;
    jsonConfigValue: Object;
    _renderStudent: Boolean;
    imageHandler: ImageHandler;
    configureSchemaJSON: Object;
    modelSchemaJSON: Object;
    textAreaContentRef1: any;
    textAreaContentRef2: any;
    cachedJsonConfig1: Object;
    cachedJsonConfig2: Object;
    rootEl: any;
    elementParent2: any;
    pieAuthor: JSX.PieAuthor;
    authorElement: any;
    piePlayer: JSX.PiePlayer;
    fileInput: any;
    elementParent1: any;
    bottomContentRef: any;
    _renderingAuthor: Boolean;
    _renderingStudent: Boolean;
    handleFileInputChange: (e: Event) => void;
    handleInsertImage: (e: InsertImageEvent) => void;
    handleDeleteImage: (e: DeleteImageEvent) => void;
    handleSetConfigElement: (e: CustomEvent) => void;
    constructor();
    componentDidUpdate(): void;
    collapsePanel(name: any): void;
    toggleAuthoringSettings(): void;
    validateItem(): Promise<void>;
    toggleStudentSettings(): void;
    isToggled(type?: string): boolean;
    detachListeners(): void;
    watchPie(newPie: any): void;
    updateModel(newModel: any): void;
    updateConfigure(newConfigure: any): Promise<void>;
    watchConfigModel(newModel: any): void;
    updatePieModelFromController(model: any, session: any, env: any, shouldResetSession?: boolean): Promise<void>;
    watchPieElement(pieElement: any): void;
    watchPieElementModel(newModel: any): void;
    handleRootEl(current: any): void;
    componentWillLoad(): void;
    handleElementResize(el: any, name: any): void;
    /**
     * Pick up @pie-framework/pie-configure-events and trigger an update.
     * TODO: Why cant i use ModelUpdatedEvent.TYPE in the decorator?
     */
    onModelUpdated(event: ModelUpdatedEvent): void;
    onModelLoaded(): void;
    onSessionChanged(event: ModelUpdatedEvent): void;
    watchConfigElement(el: PieElement): void;
    watchModelSchemaJSONURI(newUrl: any): void;
    watchConfigureSchemaJSONURI(newUrl: any): void;
    setMode(mode: any): void;
    setOption(option: any): void;
    customCheckBox({ label, checked, value, action }: {
        label: any;
        checked: any;
        value: any;
        action?: any;
    }): any;
    renderHeaderTitleInfo({ title, description, options }: {
        title: any;
        description: any;
        options?: any;
    }): any;
    viewDocumentation: (tabIndex: any) => void;
    renderJsonConfigPanel: (jsonData: any, index: any) => any;
    renderAuthoringBottomContent(): any;
    renderAuthoringHeader(smallView?: boolean): any;
    renderRoleConfigContainer(): any;
    renderModeConfigContainer(): any;
    renderSettingsContainer(): any;
    renderTabs: (tabs: any, tabWidth?: number) => any;
    renderBottomContent(): any;
    renderStudentHeader(smallView?: boolean): any;
    renderCollapsedPanel(title: any, toggled?: any): any;
    renderAuthoringHolder: (smallView?: boolean) => any;
    renderStudentHolder: (smallView?: boolean) => any;
    renderContent: () => any;
    renderDocHolder: () => any;
    render(): any;
}
export {};
